# bash completion for the Aquilon client




_aq_sqlite="/ms/dist/fsf/PROJ/sqlite/3.7.4/bin/sqlite3"
_aq_datawarehouse="/ms/dist/aquilon/PROJ/datawarehouse/dumpv3"

_aq_complete_hostname ()
{
    $_aq_sqlite $_aq_datawarehouse/hostdata.db "select hostname from hosts where hostname glob '$1*'"
}

_aq_complete_cluster ()
{
    $_aq_sqlite $_aq_datawarehouse/clusters.db "select name from cluster where name glob '$1*'"
}

_aq_complete_building ()
{
    $_aq_sqlite $_aq_datawarehouse/geography.db "select name from geography where loctype = 'building' and name glob '$1*'"
}

_aq_complete_region ()
{
    $_aq_sqlite $_aq_datawarehouse/geography.db "select name from geography where loctype = 'continent' and name glob '$1*'"
}

_aq_complete_city ()
{
    $_aq_sqlite $_aq_datawarehouse/geography.db "select name from geography where loctype = 'city' and name glob '$1*'"
}

_aq_complete_archetype ()
{
    $_aq_sqlite $_aq_datawarehouse/personality.db "select distinct archetype from personality where archetype glob '$1*'"
}

_aq_complete_personality ()
{
    $_aq_sqlite $_aq_datawarehouse/personality.db "select distinct name from personality where name glob '$1*'"
}

_aq_complete_service ()
{
    $_aq_sqlite $_aq_datawarehouse/servers.db "select distinct service from servers where service glob '$1*'"
}

_aq_complete_instance ()
{
    $_aq_sqlite $_aq_datawarehouse/servers.db "select distinct instance from servers where instance glob '$1*'"
}

_aq ()
{
    local cur cmds cmdOpts opt globalOpts globalParam optsParam optBase i

    COMPREPLY=()
    cur=${COMP_WORDS[COMP_CWORD]}

    cmds='status flush ping show_active_commands show_active_locks show_request show_host show_fqdn search_hardware search_machine search_system search_cluster search_host search_esx_cluster search_observed_mac search_next compile show_domain search_domain show_sandbox search_sandbox show_network search_network refresh_network refresh_windows_hosts add_cpu show_cpu del_cpu add_disk del_disk update_disk add_host add_aquilon_host add_windows_host add_aurora_host add_auxiliary show_auxiliary del_auxiliary add_manager show_manager del_manager add_model show_model search_model update_model del_model add_interface add_network_environment del_network_environment update_network_environment show_network_environment search_network_environment add_network del_network update_network split_network merge_network add_router del_router show_router update_router add_static_route del_static_route update_interface del_interface add_interface_address del_interface_address add_service show_service show_nas_disk_share update_service del_service bind_client rebind_client unbind_client bind_server unbind_server add_required_service show_archetype del_required_service map_service show_map unmap_service del_host del_windows_host pxeswitch reconfigure change_status reset_advertised_status add_domain update_archetype update_sandbox update_domain del_domain add_sandbox get publish deploy validate rollback del_sandbox manage cat sync add_chassis show_chassis del_chassis update_chassis add_rack update_rack show_rack del_rack add_desk show_desk del_desk add_building update_building show_building del_building add_campus del_campus add_room show_room del_room add_bunker show_bunker del_bunker show_campus add_city update_city show_city del_city add_country show_country del_country add_continent show_continent del_continent add_organization del_organization show_organization add_hub show_hub del_hub add_archetype del_archetype add_personality add_vendor del_vendor show_vendor add_os update_os del_os show_os del_personality update_personality show_personality search_personality make make_aquilon make_cluster add_machine update_machine show_machine del_machine add_switch update_switch show_switch search_switch del_switch poll_switch show_principal search_principal permission add_dns_environment show_dns_environment del_dns_environment add_dns_domain show_dns_domain del_dns_domain update_dns_domain dump_dns add_metacluster show_metacluster update_metacluster del_metacluster show_cluster add_esx_cluster show_esx_cluster update_esx_cluster del_esx_cluster add_cluster update_cluster del_cluster add_allowed_personality del_allowed_personality rebind_metacluster bind_cluster rebind_cluster unbind_cluster bind_esx_cluster rebind_esx_cluster unbind_esx_cluster cluster uncluster add_dynamic_range del_dynamic_range show_dynamic_range add_address del_address show_address update_address show_ns_record add_ns_record del_ns_record add_alias del_alias show_alias update_alias add_srv_record del_srv_record show_srv_record update_srv_record search_dns search_rack map_dns_domain unmap_dns_domain search_dns_domain_map search_audit show_reboot_schedule add_reboot_schedule del_reboot_schedule show_reboot_intervention add_reboot_intervention del_reboot_intervention add_filesystem del_filesystem show_filesystem add_share del_share show_share add_application del_application show_application add_hostlink del_hostlink show_hostlink add_intervention show_intervention del_intervention add_service_address del_service_address show_service_address add_grn del_grn update_grn show_grn refresh_grns map_grn unmap_grn add_resourcegroup del_resourcegroup show_resourcegroup add_feature del_feature show_feature bind_feature unbind_feature show_parameter validate_parameter add_parameter update_parameter del_parameter show_diff add_parameter_definition del_parameter_definition update_parameter_definition search_parameter_definition search_parameter add_vlan del_vlan show_vlan show_bunker_violations'
    globalOpts='--verbose --quiet --partialok --debug --httpinfo --format --auth --exec --usesock --slowstatus --aqport --aqhost --aquser --aqservice'
    globalParam='--verbose|--quiet|--format|--usesock|--aqport|--aqhost|--aquser|--aqservice'

    if [[ $COMP_CWORD -eq 1 ]]; then
        COMPREPLY=( $( compgen -W "$cmds" -- $cur ) )
        return 0
    fi

    if [[ $COMP_CWORD -eq 2 && ${COMP_WORDS[1]} == "help" ]]; then
        COMPREPLY=( $( compgen -W "$cmds" -- $cur ) )
        return 0
    fi

    # auto-complete some options
    if [[ $COMP_CWORD -gt 2 && ! ( $cur =~ "^-") ]]; then
        local opt full_opt subject list

	full_opt=${COMP_WORDS[ $(( $COMP_CWORD - 1 )) ]}
	opt=${full_opt/--/}

	if [[ $opt == "name" ]]; then
	    # --name is a special case, extract from the command name
	    # what we're looking for
	    opt=${COMP_WORDS[1]##*_}
	fi

	subject=""
	if typeset -F _aq_complete_${opt} >/dev/null 2>&1; then
	    subject=${opt}
	fi

	if [[ -n "$subject" ]]; then
	    list=$( _aq_complete_${opt} )
	    COMPREPLY=( $( compgen -W "$list" -- $cur ) )
	    return 0
	fi
    fi

    cmdOpts=
    case ${COMP_WORDS[1]} in

    status)
        cmdOpts=''
        optsParam=''
        ;;

    flush)
        cmdOpts='--hosts --machines --personalities --clusters --services --resources --locations --switches --all'
        optsParam='--hosts|--machines|--personalities|--clusters|--services|--resources|--locations|--switches|--all'
        ;;

    ping)
        cmdOpts=''
        optsParam=''
        ;;

    show_active_commands)
        cmdOpts=''
        optsParam=''
        ;;

    show_active_locks)
        cmdOpts=''
        optsParam=''
        ;;

    show_request)
        cmdOpts='--requestid --auditid'
        optsParam='--requestid|--auditid'
        ;;

    show_host)
        cmdOpts='--hostname --all --grns'
        optsParam='--hostname|--all|--grns'
        ;;

    show_fqdn)
        cmdOpts='--fqdn --all --dns_environment'
        optsParam='--fqdn|--all|--dns_environment'
        ;;

    search_hardware)
        cmdOpts='--model --machine_type --vendor --ip --mac --pg --interface_model --interface_vendor --serial --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --exact_location --all --fullinfo'
        optsParam='--model|--machine_type|--vendor|--ip|--mac|--pg|--interface_model|--interface_vendor|--serial|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--exact_location|--all|--fullinfo'
        ;;

    search_machine)
        cmdOpts='--hostname --machine --cpuname --cpuvendor --cpuspeed --cpucount --memory --cluster --model --machine_type --vendor --ip --mac --interface_model --interface_vendor --pg --serial --share --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --exact_location --all --fullinfo'
        optsParam='--hostname|--machine|--cpuname|--cpuvendor|--cpuspeed|--cpucount|--memory|--cluster|--model|--machine_type|--vendor|--ip|--mac|--interface_model|--interface_vendor|--pg|--serial|--share|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--exact_location|--all|--fullinfo'
        ;;

    search_system)
        cmdOpts='--fqdn --dns_domain --shortname --type --ip --networkip --mac --all --fullinfo'
        optsParam='--fqdn|--dns_domain|--shortname|--type|--ip|--networkip|--mac|--all|--fullinfo'
        ;;

    search_cluster)
        cmdOpts='--archetype --buildstatus --cluster_type --personality --allowed_archetype --allowed_personality --down_hosts_threshold --down_maint_threshold --max_members --member_archetype --member_hostname --member_personality --domain --sandbox --branch --cluster --esx_metacluster --esx_virtual_machine --esx_guest --service --instance --esx_share --share --capacity_override --esx_switch --cluster_organization --cluster_hub --cluster_continent --cluster_country --cluster_campus --cluster_city --cluster_building --cluster_room --cluster_bunker --cluster_rack --cluster_desk --cluster_exact_location --member_organization --member_hub --member_continent --member_country --member_campus --member_city --member_building --member_room --member_bunker --member_rack --member_desk --member_exact_location --all --fullinfo'
        optsParam='--archetype|--buildstatus|--cluster_type|--personality|--allowed_archetype|--allowed_personality|--down_hosts_threshold|--down_maint_threshold|--max_members|--member_archetype|--member_hostname|--member_personality|--domain|--sandbox|--branch|--cluster|--esx_metacluster|--esx_virtual_machine|--esx_guest|--service|--instance|--esx_share|--share|--capacity_override|--esx_switch|--cluster_organization|--cluster_hub|--cluster_continent|--cluster_country|--cluster_campus|--cluster_city|--cluster_building|--cluster_room|--cluster_bunker|--cluster_rack|--cluster_desk|--cluster_exact_location|--member_organization|--member_hub|--member_continent|--member_country|--member_campus|--member_city|--member_building|--member_room|--member_bunker|--member_rack|--member_desk|--member_exact_location|--all|--fullinfo'
        ;;

    search_host)
        cmdOpts='--hostname --machine --dns_domain --shortname --archetype --buildstatus --ip --networkip --network_environment --mac --personality --osname --osversion --service --instance --model --machine_type --vendor --serial --cluster --guest_on_cluster --guest_on_share --member_cluster_share --domain --sandbox --branch --sandbox_owner --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --server_of_service --server_of_instance --exact_location --grn --eon_id --all --fullinfo'
        optsParam='--hostname|--machine|--dns_domain|--shortname|--archetype|--buildstatus|--ip|--networkip|--network_environment|--mac|--personality|--osname|--osversion|--service|--instance|--model|--machine_type|--vendor|--serial|--cluster|--guest_on_cluster|--guest_on_share|--member_cluster_share|--domain|--sandbox|--branch|--sandbox_owner|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--server_of_service|--server_of_instance|--exact_location|--grn|--eon_id|--all|--fullinfo'
        ;;

    search_esx_cluster)
        cmdOpts='--cluster --metacluster --esx_hostname --virtual_machine --guest --archetype --personality --service --instance --share --capacity_override --switch --buildstatus --domain --sandbox --branch --cluster_organization --cluster_hub --cluster_continent --cluster_country --cluster_campus --cluster_city --cluster_building --cluster_room --cluster_bunker --cluster_rack --cluster_desk --cluster_exact_location --vmhost_organization --vmhost_hub --vmhost_continent --vmhost_country --vmhost_campus --vmhost_city --vmhost_building --vmhost_room --vmhost_bunker --vmhost_rack --vmhost_desk --vmhost_exact_location --all --fullinfo'
        optsParam='--cluster|--metacluster|--esx_hostname|--virtual_machine|--guest|--archetype|--personality|--service|--instance|--share|--capacity_override|--switch|--buildstatus|--domain|--sandbox|--branch|--cluster_organization|--cluster_hub|--cluster_continent|--cluster_country|--cluster_campus|--cluster_city|--cluster_building|--cluster_room|--cluster_bunker|--cluster_rack|--cluster_desk|--cluster_exact_location|--vmhost_organization|--vmhost_hub|--vmhost_continent|--vmhost_country|--vmhost_campus|--vmhost_city|--vmhost_building|--vmhost_room|--vmhost_bunker|--vmhost_rack|--vmhost_desk|--vmhost_exact_location|--all|--fullinfo'
        ;;

    search_observed_mac)
        cmdOpts='--switch --port --mac'
        optsParam='--switch|--port|--mac'
        ;;

    search_next)
        cmdOpts='--machine --short --dns_domain --cluster --metacluster --number --pack --start'
        optsParam='--machine|--short|--dns_domain|--cluster|--metacluster|--number|--pack|--start'
        ;;

    compile)
        cmdOpts='--domain --sandbox --hostname --cluster --personality --archetype --pancinclude --pancexclude --pancdebug --cleandeps'
        optsParam='--domain|--sandbox|--hostname|--cluster|--personality|--archetype|--pancinclude|--pancexclude'
        ;;

    show_domain)
        cmdOpts='--domain --all'
        optsParam='--domain|--all'
        ;;

    search_domain)
        cmdOpts='--track --not_tracking --change_manager --owner --compiler_version --autosync --validated --fullinfo'
        optsParam='--track|--not_tracking|--owner|--compiler_version|--fullinfo'
        ;;

    show_sandbox)
        cmdOpts='--sandbox --all --pathonly'
        optsParam='--sandbox|--all|--pathonly'
        ;;

    search_sandbox)
        cmdOpts='--owner --compiler_version --autosync --validated --fullinfo'
        optsParam='--owner|--compiler_version|--fullinfo'
        ;;

    show_network)
        cmdOpts='--network --ip --all --type --hosts --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --exact_location --network_environment'
        optsParam='--network|--ip|--all|--type|--hosts|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--exact_location|--network_environment'
        ;;

    search_network)
        cmdOpts='--network --ip --type --side --machine --fqdn --cluster --pg --has_dynamic_ranges --exact_location --all --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --network_environment --fullinfo'
        optsParam='--network|--ip|--type|--side|--machine|--fqdn|--cluster|--pg|--has_dynamic_ranges|--exact_location|--all|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--network_environment|--fullinfo'
        ;;

    refresh_network)
        cmdOpts='--building --all --dryrun --incremental'
        optsParam='--building|--all'
        ;;

    refresh_windows_hosts)
        cmdOpts='--dryrun'
        optsParam=''
        ;;

    add_cpu)
        cmdOpts='--cpu --vendor --speed --comments'
        optsParam='--cpu|--vendor|--speed|--comments'
        ;;

    show_cpu)
        cmdOpts='--all --cpu --vendor --speed'
        optsParam='--all|--cpu|--vendor|--speed'
        ;;

    del_cpu)
        cmdOpts='--cpu --vendor --speed'
        optsParam='--cpu|--vendor|--speed'
        ;;

    add_disk)
        cmdOpts='--machine --disk --controller --size --share --filesystem --resourcegroup --address --comments --boot --snapshot'
        optsParam='--machine|--disk|--controller|--size|--share|--filesystem|--resourcegroup|--address|--comments'
        ;;

    del_disk)
        cmdOpts='--machine --disk --controller --size --all'
        optsParam='--machine|--disk|--controller|--size|--all'
        ;;

    update_disk)
        cmdOpts='--machine --disk --rename_to --share --filesystem --resourcegroup --address --controller --size --comments --boot --snapshot'
        optsParam='--machine|--disk|--rename_to|--share|--filesystem|--resourcegroup|--address|--controller|--size|--comments'
        ;;

    add_host)
        cmdOpts='--hostname --prefix --machine --archetype --domain --sandbox --ip --ipfromip --ipfromsystem --autoip --osname --osversion --buildstatus --personality --ipalgorithm --zebra_interfaces --comments --grn --eon_id --dns_domain'
        optsParam='--hostname|--prefix|--machine|--archetype|--domain|--sandbox|--ip|--ipfromip|--ipfromsystem|--autoip|--osname|--osversion|--buildstatus|--personality|--ipalgorithm|--zebra_interfaces|--comments|--grn|--eon_id|--dns_domain'
        ;;

    add_aquilon_host)
        cmdOpts='--hostname --machine --domain --sandbox --ip --ipfromip --ipfromsystem --autoip --osname --osversion --buildstatus --comments --personality --ipalgorithm --zebra_interfaces --grn --eon_id'
        optsParam='--hostname|--machine|--domain|--sandbox|--ip|--ipfromip|--ipfromsystem|--autoip|--osname|--osversion|--buildstatus|--comments|--personality|--ipalgorithm|--zebra_interfaces|--grn|--eon_id'
        ;;

    add_windows_host)
        cmdOpts='--hostname --machine --buildstatus --personality --osversion --comments --ipalgorithm --ip --ipfromip --ipfromsystem --autoip --grn --eon_id'
        optsParam='--hostname|--machine|--buildstatus|--personality|--osversion|--comments|--ipalgorithm|--ip|--ipfromip|--ipfromsystem|--autoip|--grn|--eon_id'
        ;;

    add_aurora_host)
        cmdOpts='--hostname --osname --osversion --comments'
        optsParam='--hostname|--osname|--osversion|--comments'
        ;;

    add_auxiliary)
        cmdOpts='--hostname --machine --auxiliary --ip --ipfromip --ipfromsystem --autoip --interface --mac --ipalgorithm --comments'
        optsParam='--hostname|--machine|--auxiliary|--ip|--ipfromip|--ipfromsystem|--autoip|--interface|--mac|--ipalgorithm|--comments'
        ;;

    show_auxiliary)
        cmdOpts='--auxiliary --all'
        optsParam='--auxiliary|--all'
        ;;

    del_auxiliary)
        cmdOpts='--auxiliary'
        optsParam='--auxiliary'
        ;;

    add_manager)
        cmdOpts='--hostname --ip --ipfromip --ipfromsystem --autoip --manager --interface --mac --ipalgorithm --comments'
        optsParam='--hostname|--ip|--ipfromip|--ipfromsystem|--autoip|--manager|--interface|--mac|--ipalgorithm|--comments'
        ;;

    show_manager)
        cmdOpts='--manager --all'
        optsParam='--manager|--all'
        ;;

    del_manager)
        cmdOpts='--manager'
        optsParam='--manager'
        ;;

    add_model)
        cmdOpts='--model --vendor --type --cpuname --cpunum --memory --disktype --diskcontroller --disksize --nics --cpuvendor --cpuspeed --nicmodel --nicvendor --comments'
        optsParam='--model|--vendor|--type|--cpuname|--cpunum|--memory|--disktype|--diskcontroller|--disksize|--nics|--cpuvendor|--cpuspeed|--nicmodel|--nicvendor|--comments'
        ;;

    show_model)
        cmdOpts='--model --vendor --type --all'
        optsParam='--model|--vendor|--type|--all'
        ;;

    search_model)
        cmdOpts='--model --vendor --machine_type --cpuname --cpuvendor --cpuspeed --cpunum --nicmodel --nicvendor --memory --disktype --diskcontroller --disksize --fullinfo'
        optsParam='--model|--vendor|--machine_type|--cpuname|--cpuvendor|--cpuspeed|--cpunum|--nicmodel|--nicvendor|--memory|--disktype|--diskcontroller|--disksize|--fullinfo'
        ;;

    update_model)
        cmdOpts='--model --vendor --newmodel --newvendor --machine_type --cpuname --cpuvendor --cpuspeed --cpunum --memory --disktype --diskcontroller --disksize --nics --nicmodel --nicvendor --comments --leave_existing'
        optsParam='--model|--vendor|--newmodel|--newvendor|--machine_type|--cpuname|--cpuvendor|--cpuspeed|--cpunum|--memory|--disktype|--diskcontroller|--disksize|--nics|--nicmodel|--nicvendor|--comments'
        ;;

    del_model)
        cmdOpts='--model --vendor'
        optsParam='--model|--vendor'
        ;;

    add_interface)
        cmdOpts='--interface --mac --automac --machine --hostname --chassis --switch --type --comments --pg --autopg --model --vendor'
        optsParam='--interface|--mac|--automac|--machine|--hostname|--chassis|--switch|--type|--comments|--pg|--autopg|--model|--vendor'
        ;;

    add_network_environment)
        cmdOpts='--network_environment --dns_environment --building --comments'
        optsParam='--network_environment|--dns_environment|--building|--comments'
        ;;

    del_network_environment)
        cmdOpts='--network_environment'
        optsParam='--network_environment'
        ;;

    update_network_environment)
        cmdOpts='--network_environment --building --clear_location --comments'
        optsParam='--network_environment|--building|--clear_location|--comments'
        ;;

    show_network_environment)
        cmdOpts='--network_environment --all'
        optsParam='--network_environment|--all'
        ;;

    search_network_environment)
        cmdOpts='--network_environment --building'
        optsParam='--network_environment|--building'
        ;;

    add_network)
        cmdOpts='--network --ip --netmask --prefixlen --mask --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --network_environment --type --side --comments'
        optsParam='--network|--ip|--netmask|--prefixlen|--mask|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--network_environment|--type|--side|--comments'
        ;;

    del_network)
        cmdOpts='--ip --network_environment'
        optsParam='--ip|--network_environment'
        ;;

    update_network)
        cmdOpts='--network --ip --type --side --comments --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --network_environment'
        optsParam='--network|--ip|--type|--side|--comments|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--network_environment'
        ;;

    split_network)
        cmdOpts='--ip --netmask --prefixlen --network_environment'
        optsParam='--ip|--netmask|--prefixlen|--network_environment'
        ;;

    merge_network)
        cmdOpts='--ip --netmask --prefixlen --network_environment'
        optsParam='--ip|--netmask|--prefixlen|--network_environment'
        ;;

    add_router)
        cmdOpts='--fqdn --network_environment --building --ip --comments'
        optsParam='--fqdn|--network_environment|--building|--ip|--comments'
        ;;

    del_router)
        cmdOpts='--fqdn --ip --network_environment'
        optsParam='--fqdn|--ip|--network_environment'
        ;;

    show_router)
        cmdOpts='--fqdn --ip --all --network_environment'
        optsParam='--fqdn|--ip|--all|--network_environment'
        ;;

    update_router)
        cmdOpts='--fqdn --ip --building --comments'
        optsParam='--fqdn|--ip|--building|--comments'
        ;;

    add_static_route)
        cmdOpts='--ip --netmask --prefixlen --gateway --networkip --network_environment --comments'
        optsParam='--ip|--netmask|--prefixlen|--gateway|--networkip|--network_environment|--comments'
        ;;

    del_static_route)
        cmdOpts='--gateway --ip --netmask --prefixlen --network_environment'
        optsParam='--gateway|--ip|--netmask|--prefixlen|--network_environment'
        ;;

    update_interface)
        cmdOpts='--interface --machine --hostname --switch --chassis --mac --boot --default_route --pg --autopg --comments --master --clear_master --model --vendor --rename_to'
        optsParam='--interface|--machine|--hostname|--switch|--chassis|--mac|--boot|--pg|--autopg|--comments|--master|--clear_master|--model|--vendor|--rename_to'
        ;;

    del_interface)
        cmdOpts='--interface --machine --switch --chassis --mac'
        optsParam='--interface|--machine|--switch|--chassis|--mac'
        ;;

    add_interface_address)
        cmdOpts='--machine --chassis --switch --interface --fqdn --network_environment --label --ipalgorithm --map_to_primary --ip --ipfromip --ipfromsystem --autoip'
        optsParam='--machine|--chassis|--switch|--interface|--fqdn|--network_environment|--label|--ipalgorithm|--ip|--ipfromip|--ipfromsystem|--autoip'
        ;;

    del_interface_address)
        cmdOpts='--machine --chassis --switch --interface --keep_dns --network_environment --fqdn --ip --label'
        optsParam='--machine|--chassis|--switch|--interface|--keep_dns|--network_environment|--fqdn|--ip|--label'
        ;;

    add_service)
        cmdOpts='--service --comments --instance --need_client_list'
        optsParam='--service|--comments|--instance'
        ;;

    show_service)
        cmdOpts='--service --server --client --all --instance'
        optsParam='--service|--server|--client|--all|--instance'
        ;;

    show_nas_disk_share)
        cmdOpts='--share --all'
        optsParam='--share|--all'
        ;;

    update_service)
        cmdOpts='--service --max_clients --default --need_client_list --instance --comments'
        optsParam='--service|--max_clients|--default|--instance|--comments'
        ;;

    del_service)
        cmdOpts='--service --instance'
        optsParam='--service|--instance'
        ;;

    bind_client)
        cmdOpts='--hostname --service --instance'
        optsParam='--hostname|--service|--instance'
        ;;

    rebind_client)
        cmdOpts='--hostname --service --instance'
        optsParam='--hostname|--service|--instance'
        ;;

    unbind_client)
        cmdOpts='--hostname --service'
        optsParam='--hostname|--service'
        ;;

    bind_server)
        cmdOpts='--service --instance --hostname --cluster --alias --ip --service_address --resourcegroup --position'
        optsParam='--service|--instance|--hostname|--cluster|--alias|--ip|--service_address|--resourcegroup|--position'
        ;;

    unbind_server)
        cmdOpts='--service --instance --all --position --hostname --cluster --alias --ip --service_address --resourcegroup'
        optsParam='--service|--instance|--all|--position|--hostname|--cluster|--alias|--ip|--service_address|--resourcegroup'
        ;;

    add_required_service)
        cmdOpts='--service --archetype --personality --justification'
        optsParam='--service|--archetype|--personality|--justification'
        ;;

    show_archetype)
        cmdOpts='--archetype --all'
        optsParam='--archetype|--all'
        ;;

    del_required_service)
        cmdOpts='--service --archetype --personality --justification'
        optsParam='--service|--archetype|--personality|--justification'
        ;;

    map_service)
        cmdOpts='--service --instance --archetype --personality --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --networkip'
        optsParam='--service|--instance|--archetype|--personality|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--networkip'
        ;;

    show_map)
        cmdOpts='--service --instance --archetype --personality --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --networkip --all --include_parents'
        optsParam='--service|--instance|--archetype|--personality|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--networkip|--all|--include_parents'
        ;;

    unmap_service)
        cmdOpts='--service --instance --archetype --personality --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --networkip'
        optsParam='--service|--instance|--archetype|--personality|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--networkip'
        ;;

    del_host)
        cmdOpts='--hostname'
        optsParam='--hostname'
        ;;

    del_windows_host)
        cmdOpts='--hostname'
        optsParam='--hostname'
        ;;

    pxeswitch)
        cmdOpts='--hostname --list --status --configure --localboot --install --firmware --blindbuild --rescue'
        optsParam='--hostname|--list|--status|--localboot|--install|--firmware|--blindbuild|--rescue'
        ;;

    reconfigure)
        cmdOpts='--hostname --hostlist --list --membersof --osname --osversion --archetype --personality --buildstatus --keepbindings --grn --eon_id --cleargrn'
        optsParam='--hostname|--hostlist|--list|--membersof|--osname|--osversion|--archetype|--personality|--buildstatus|--grn|--eon_id|--cleargrn'
        ;;

    change_status)
        cmdOpts='--hostname --cluster --buildstatus'
        optsParam='--hostname|--cluster|--buildstatus'
        ;;

    reset_advertised_status)
        cmdOpts='--hostname --list'
        optsParam='--hostname|--list'
        ;;

    add_domain)
        cmdOpts='--domain --track --start --comments --change_manager --allow_manage'
        optsParam='--domain|--track|--start|--comments'
        ;;

    update_archetype)
        cmdOpts='--archetype --compilable --description --cluster_type'
        optsParam='--archetype|--description|--cluster_type'
        ;;

    update_sandbox)
        cmdOpts='--sandbox --comments --compiler_version --autosync'
        optsParam='--sandbox|--comments|--compiler_version'
        ;;

    update_domain)
        cmdOpts='--domain --comments --compiler_version --autosync --change_manager --allow_manage'
        optsParam='--domain|--comments|--compiler_version'
        ;;

    del_domain)
        cmdOpts='--domain'
        optsParam='--domain'
        ;;

    add_sandbox)
        cmdOpts='--sandbox --start --get --comments'
        optsParam='--sandbox|--start|--comments'
        ;;

    get)
        cmdOpts='--sandbox'
        optsParam='--sandbox'
        ;;

    publish)
        cmdOpts='--branch --sandbox --sync --rebase'
        optsParam='--branch|--sandbox|--rebase'
        ;;

    deploy)
        cmdOpts='--source --target --sync --dryrun --comments --justification'
        optsParam='--source|--target|--comments|--justification'
        ;;

    validate)
        cmdOpts='--branch --comments'
        optsParam='--branch|--comments'
        ;;

    rollback)
        cmdOpts='--domain --ref --lastsync'
        optsParam='--domain|--ref|--lastsync'
        ;;

    del_sandbox)
        cmdOpts='--sandbox'
        optsParam='--sandbox'
        ;;

    manage)
        cmdOpts='--hostname --cluster --list --domain --sandbox --force'
        optsParam='--hostname|--cluster|--list|--domain|--sandbox|--force'
        ;;

    cat)
        cmdOpts='--machine --hostname --cluster --service --city --switch --personality --archetype --pre_feature --post_feature --param_tmpl --default --server --data --instance --generate --resourcegroup --application --filesystem --hostlink --intervention --reboot_intervention --reboot_schedule --virtual_machine --service_address --share'
        optsParam='--machine|--hostname|--cluster|--service|--city|--switch|--personality|--archetype|--pre_feature|--post_feature|--param_tmpl|--default|--server|--data|--instance|--generate|--resourcegroup|--application|--filesystem|--hostlink|--intervention|--reboot_intervention|--reboot_schedule|--virtual_machine|--service_address|--share'
        ;;

    sync)
        cmdOpts='--domain'
        optsParam='--domain'
        ;;

    add_chassis)
        cmdOpts='--chassis --rack --model --ip --interface --mac --label --vendor --serial --comments'
        optsParam='--chassis|--rack|--model|--ip|--interface|--mac|--label|--vendor|--serial|--comments'
        ;;

    show_chassis)
        cmdOpts='--chassis --all'
        optsParam='--chassis|--all'
        ;;

    del_chassis)
        cmdOpts='--chassis --clear_slots'
        optsParam='--chassis|--clear_slots'
        ;;

    update_chassis)
        cmdOpts='--chassis --model --rack --ip --vendor --serial --comments'
        optsParam='--chassis|--model|--rack|--ip|--vendor|--serial|--comments'
        ;;

    add_rack)
        cmdOpts='--rackid --row --column --building --room --bunker --fullname --comments'
        optsParam='--rackid|--row|--column|--building|--room|--bunker|--fullname|--comments'
        ;;

    update_rack)
        cmdOpts='--rack --row --column --building --room --bunker --fullname --comments --default_dns_domain'
        optsParam='--rack|--row|--column|--building|--room|--bunker|--fullname|--comments|--default_dns_domain'
        ;;

    show_rack)
        cmdOpts='--rack --all'
        optsParam='--rack|--all'
        ;;

    del_rack)
        cmdOpts='--rack'
        optsParam='--rack'
        ;;

    add_desk)
        cmdOpts='--desk --building --room --fullname --comments'
        optsParam='--desk|--building|--room|--fullname|--comments'
        ;;

    show_desk)
        cmdOpts='--desk --all'
        optsParam='--desk|--all'
        ;;

    del_desk)
        cmdOpts='--desk'
        optsParam='--desk'
        ;;

    add_building)
        cmdOpts='--building --address --city --fullname --comments'
        optsParam='--building|--address|--city|--fullname|--comments'
        ;;

    update_building)
        cmdOpts='--building --address --city --fullname --default_dns_domain --comments'
        optsParam='--building|--address|--city|--fullname|--default_dns_domain|--comments'
        ;;

    show_building)
        cmdOpts='--building --all'
        optsParam='--building|--all'
        ;;

    del_building)
        cmdOpts='--building'
        optsParam='--building'
        ;;

    add_campus)
        cmdOpts='--campus --country --fullname --comments'
        optsParam='--campus|--country|--fullname|--comments'
        ;;

    del_campus)
        cmdOpts='--campus'
        optsParam='--campus'
        ;;

    add_room)
        cmdOpts='--room --building --fullname --comments'
        optsParam='--room|--building|--fullname|--comments'
        ;;

    show_room)
        cmdOpts='--room --all'
        optsParam='--room|--all'
        ;;

    del_room)
        cmdOpts='--room'
        optsParam='--room'
        ;;

    add_bunker)
        cmdOpts='--bunker --room --building --fullname --comments'
        optsParam='--bunker|--room|--building|--fullname|--comments'
        ;;

    show_bunker)
        cmdOpts='--bunker --all'
        optsParam='--bunker|--all'
        ;;

    del_bunker)
        cmdOpts='--bunker'
        optsParam='--bunker'
        ;;

    show_campus)
        cmdOpts='--campus --all'
        optsParam='--campus|--all'
        ;;

    add_city)
        cmdOpts='--city --timezone --country --campus --fullname --comments'
        optsParam='--city|--timezone|--country|--campus|--fullname|--comments'
        ;;

    update_city)
        cmdOpts='--city --timezone --campus --default_dns_domain --comments'
        optsParam='--city|--timezone|--campus|--default_dns_domain|--comments'
        ;;

    show_city)
        cmdOpts='--city --all'
        optsParam='--city|--all'
        ;;

    del_city)
        cmdOpts='--city'
        optsParam='--city'
        ;;

    add_country)
        cmdOpts='--country --continent --fullname --comments'
        optsParam='--country|--continent|--fullname|--comments'
        ;;

    show_country)
        cmdOpts='--country --all'
        optsParam='--country|--all'
        ;;

    del_country)
        cmdOpts='--country'
        optsParam='--country'
        ;;

    add_continent)
        cmdOpts='--continent --hub --fullname --comments'
        optsParam='--continent|--hub|--fullname|--comments'
        ;;

    show_continent)
        cmdOpts='--continent --all'
        optsParam='--continent|--all'
        ;;

    del_continent)
        cmdOpts='--continent'
        optsParam='--continent'
        ;;

    add_organization)
        cmdOpts='--organization --fullname --comments'
        optsParam='--organization|--fullname|--comments'
        ;;

    del_organization)
        cmdOpts='--organization'
        optsParam='--organization'
        ;;

    show_organization)
        cmdOpts='--organization --all'
        optsParam='--organization|--all'
        ;;

    add_hub)
        cmdOpts='--hub --fullname --organization --comments'
        optsParam='--hub|--fullname|--organization|--comments'
        ;;

    show_hub)
        cmdOpts='--hub --all'
        optsParam='--hub|--all'
        ;;

    del_hub)
        cmdOpts='--hub'
        optsParam='--hub'
        ;;

    add_archetype)
        cmdOpts='--archetype --compilable --description --cluster_type'
        optsParam='--archetype|--description|--cluster_type'
        ;;

    del_archetype)
        cmdOpts='--archetype'
        optsParam='--archetype'
        ;;

    add_personality)
        cmdOpts='--personality --archetype --grn --eon_id --host_environment --cluster_required --comments --copy_from --config_override'
        optsParam='--personality|--archetype|--grn|--eon_id|--host_environment|--comments|--copy_from'
        ;;

    add_vendor)
        cmdOpts='--vendor --comments'
        optsParam='--vendor|--comments'
        ;;

    del_vendor)
        cmdOpts='--vendor'
        optsParam='--vendor'
        ;;

    show_vendor)
        cmdOpts='--vendor --all'
        optsParam='--vendor|--all'
        ;;

    add_os)
        cmdOpts='--osname --osversion --archetype --comments'
        optsParam='--osname|--osversion|--archetype|--comments'
        ;;

    update_os)
        cmdOpts='--osname --osversion --archetype --comments'
        optsParam='--osname|--osversion|--archetype|--comments'
        ;;

    del_os)
        cmdOpts='--osname --osversion --archetype'
        optsParam='--osname|--osversion|--archetype'
        ;;

    show_os)
        cmdOpts='--osname --osversion --archetype --all'
        optsParam='--osname|--osversion|--archetype|--all'
        ;;

    del_personality)
        cmdOpts='--personality --archetype'
        optsParam='--personality|--archetype'
        ;;

    update_personality)
        cmdOpts='--personality --archetype --vmhost_capacity_function --vmhost_overcommit_memory --cluster_required --config_override --host_environment --grn --eon_id --leave_existing'
        optsParam='--personality|--archetype|--vmhost_capacity_function|--vmhost_overcommit_memory|--host_environment|--grn|--eon_id|--leave_existing'
        ;;

    show_personality)
        cmdOpts='--personality --archetype --all --domain --sandbox'
        optsParam='--personality|--archetype|--all|--domain|--sandbox'
        ;;

    search_personality)
        cmdOpts='--personality --archetype --grn --eon_id --config_override --host_environment --fullinfo'
        optsParam='--personality|--archetype|--grn|--eon_id|--host_environment|--fullinfo'
        ;;

    make)
        cmdOpts='--hostname --osname --osversion --archetype --personality --buildstatus --keepbindings --grn --eon_id --cleargrn'
        optsParam='--hostname|--osname|--osversion|--archetype|--personality|--buildstatus|--grn|--eon_id|--cleargrn'
        ;;

    make_aquilon)
        cmdOpts='--hostname --osname --osversion --personality --buildstatus --keepbindings --grn --eon_id --cleargrn'
        optsParam='--hostname|--osname|--osversion|--personality|--buildstatus|--grn|--eon_id|--cleargrn'
        ;;

    make_cluster)
        cmdOpts='--cluster --keepbindings'
        optsParam='--cluster'
        ;;

    add_machine)
        cmdOpts='--model --machine --prefix --vendor --cpuname --cpuvendor --cpuspeed --cpucount --memory --chassis --slot --rack --desk --cluster --vmhost --serial --comments --uri'
        optsParam='--model|--machine|--prefix|--vendor|--cpuname|--cpuvendor|--cpuspeed|--cpucount|--memory|--chassis|--slot|--rack|--desk|--cluster|--vmhost|--serial|--comments|--uri'
        ;;

    update_machine)
        cmdOpts='--machine --hostname --model --vendor --cpuname --cpuvendor --cpuspeed --cpucount --memory --chassis --slot --clearchassis --multislot --rack --desk --serial --cluster --allow_metacluster_change --vmhost --ip --uri'
        optsParam='--machine|--hostname|--model|--vendor|--cpuname|--cpuvendor|--cpuspeed|--cpucount|--memory|--chassis|--slot|--rack|--desk|--serial|--cluster|--vmhost|--ip|--uri'
        ;;

    show_machine)
        cmdOpts='--machine --all --chassis --slot --rack --desk --model --vendor --machine_type'
        optsParam='--machine|--all|--chassis|--slot|--rack|--desk|--model|--vendor|--machine_type'
        ;;

    del_machine)
        cmdOpts='--machine'
        optsParam='--machine'
        ;;

    add_switch)
        cmdOpts='--switch --model --rack --type --ip --interface --mac --label --vendor --serial --comments'
        optsParam='--switch|--model|--rack|--type|--ip|--interface|--mac|--label|--vendor|--serial|--comments'
        ;;

    update_switch)
        cmdOpts='--switch --model --rack --type --ip --vendor --serial --comments --rename_to --discovered_macs --clear --discover'
        optsParam='--switch|--model|--rack|--type|--ip|--vendor|--serial|--comments|--rename_to|--discovered_macs|--clear|--discover'
        ;;

    show_switch)
        cmdOpts='--switch --all --discover'
        optsParam='--switch|--all|--discover'
        ;;

    search_switch)
        cmdOpts='--switch --ip --vlan --type --model --vendor --organization --hub --continent --country --city --building --room --bunker --rack --desk --serial --exact_location --fullinfo --all'
        optsParam='--switch|--ip|--vlan|--type|--model|--vendor|--organization|--hub|--continent|--country|--city|--building|--room|--bunker|--rack|--desk|--serial|--exact_location|--fullinfo|--all'
        ;;

    del_switch)
        cmdOpts='--switch'
        optsParam='--switch'
        ;;

    poll_switch)
        cmdOpts='--switch --rack --type --clear --vlan'
        optsParam='--switch|--rack|--type'
        ;;

    show_principal)
        cmdOpts='--principal --all'
        optsParam='--principal|--all'
        ;;

    search_principal)
        cmdOpts='--role --realm --fullinfo'
        optsParam='--role|--realm|--fullinfo'
        ;;

    permission)
        cmdOpts='--principal --role --createuser --createrealm --comments'
        optsParam='--principal|--role|--comments'
        ;;

    add_dns_environment)
        cmdOpts='--dns_environment --comments'
        optsParam='--dns_environment|--comments'
        ;;

    show_dns_environment)
        cmdOpts='--dns_environment --all'
        optsParam='--dns_environment|--all'
        ;;

    del_dns_environment)
        cmdOpts='--dns_environment'
        optsParam='--dns_environment'
        ;;

    add_dns_domain)
        cmdOpts='--dns_domain --comments --restricted'
        optsParam='--dns_domain|--comments|--restricted'
        ;;

    show_dns_domain)
        cmdOpts='--dns_domain --all'
        optsParam='--dns_domain|--all'
        ;;

    del_dns_domain)
        cmdOpts='--dns_domain'
        optsParam='--dns_domain'
        ;;

    update_dns_domain)
        cmdOpts='--dns_domain --comments --restricted'
        optsParam='--dns_domain|--comments'
        ;;

    dump_dns)
        cmdOpts='--dns_domain --dns_environment'
        optsParam='--dns_domain|--dns_environment'
        ;;

    add_metacluster)
        cmdOpts='--metacluster --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --archetype --personality --domain --sandbox --max_members --buildstatus --comments'
        optsParam='--metacluster|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--archetype|--personality|--domain|--sandbox|--max_members|--buildstatus|--comments'
        ;;

    show_metacluster)
        cmdOpts='--metacluster --all'
        optsParam='--metacluster|--all'
        ;;

    update_metacluster)
        cmdOpts='--metacluster --max_members --comments --high_availability --fix_location --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk'
        optsParam='--metacluster|--max_members|--comments|--fix_location|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk'
        ;;

    del_metacluster)
        cmdOpts='--metacluster'
        optsParam='--metacluster'
        ;;

    show_cluster)
        cmdOpts='--cluster --all'
        optsParam='--cluster|--all'
        ;;

    add_esx_cluster)
        cmdOpts='--cluster --metacluster --down_hosts_threshold --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --archetype --personality --domain --sandbox --maint_threshold --max_members --vm_to_host_ratio --switch --buildstatus --comments'
        optsParam='--cluster|--metacluster|--down_hosts_threshold|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--archetype|--personality|--domain|--sandbox|--maint_threshold|--max_members|--vm_to_host_ratio|--switch|--buildstatus|--comments'
        ;;

    show_esx_cluster)
        cmdOpts='--cluster --all'
        optsParam='--cluster|--all'
        ;;

    update_esx_cluster)
        cmdOpts='--cluster --personality --down_hosts_threshold --maint_threshold --max_members --vm_to_host_ratio --switch --fix_location --comments --memory_capacity --clear_overrides --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk'
        optsParam='--cluster|--personality|--down_hosts_threshold|--maint_threshold|--max_members|--vm_to_host_ratio|--switch|--fix_location|--comments|--memory_capacity|--clear_overrides|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk'
        ;;

    del_esx_cluster)
        cmdOpts='--cluster'
        optsParam='--cluster'
        ;;

    add_cluster)
        cmdOpts='--cluster --archetype --personality --down_hosts_threshold --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --domain --sandbox --metacluster --maint_threshold --max_members --vm_to_host_ratio --switch --buildstatus --comments'
        optsParam='--cluster|--archetype|--personality|--down_hosts_threshold|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--domain|--sandbox|--metacluster|--maint_threshold|--max_members|--vm_to_host_ratio|--switch|--buildstatus|--comments'
        ;;

    update_cluster)
        cmdOpts='--cluster --personality --down_hosts_threshold --maint_threshold --max_members --vm_to_host_ratio --switch --fix_location --comments --memory_capacity --clear_overrides --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk'
        optsParam='--cluster|--personality|--down_hosts_threshold|--maint_threshold|--max_members|--vm_to_host_ratio|--switch|--fix_location|--comments|--memory_capacity|--clear_overrides|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk'
        ;;

    del_cluster)
        cmdOpts='--cluster'
        optsParam='--cluster'
        ;;

    add_allowed_personality)
        cmdOpts='--cluster --archetype --personality'
        optsParam='--cluster|--archetype|--personality'
        ;;

    del_allowed_personality)
        cmdOpts='--cluster --archetype --personality'
        optsParam='--cluster|--archetype|--personality'
        ;;

    rebind_metacluster)
        cmdOpts='--metacluster --cluster'
        optsParam='--metacluster|--cluster'
        ;;

    bind_cluster)
        cmdOpts='--cluster --service --instance'
        optsParam='--cluster|--service|--instance'
        ;;

    rebind_cluster)
        cmdOpts='--cluster --service --instance'
        optsParam='--cluster|--service|--instance'
        ;;

    unbind_cluster)
        cmdOpts='--cluster --service --instance'
        optsParam='--cluster|--service|--instance'
        ;;

    bind_esx_cluster)
        cmdOpts='--cluster --service --instance'
        optsParam='--cluster|--service|--instance'
        ;;

    rebind_esx_cluster)
        cmdOpts='--cluster --service --instance'
        optsParam='--cluster|--service|--instance'
        ;;

    unbind_esx_cluster)
        cmdOpts='--cluster --service --instance'
        optsParam='--cluster|--service|--instance'
        ;;

    cluster)
        cmdOpts='--hostname --cluster --personality'
        optsParam='--hostname|--cluster|--personality'
        ;;

    uncluster)
        cmdOpts='--hostname --cluster --personality'
        optsParam='--hostname|--cluster|--personality'
        ;;

    add_dynamic_range)
        cmdOpts='--dns_domain --fillnetwork --startip --endip --prefix'
        optsParam='--dns_domain|--fillnetwork|--startip|--endip|--prefix'
        ;;

    del_dynamic_range)
        cmdOpts='--clearnetwork --startip --endip'
        optsParam='--clearnetwork|--startip|--endip'
        ;;

    show_dynamic_range)
        cmdOpts='--ip --fqdn --dns_environment'
        optsParam='--ip|--fqdn|--dns_environment'
        ;;

    add_address)
        cmdOpts='--fqdn --ip --ipfromip --ipfromsystem --dns_environment --network_environment --ipalgorithm --comments --reverse_ptr'
        optsParam='--fqdn|--ip|--ipfromip|--ipfromsystem|--dns_environment|--network_environment|--ipalgorithm|--comments|--reverse_ptr'
        ;;

    del_address)
        cmdOpts='--fqdn --ip --dns_environment --network_environment'
        optsParam='--fqdn|--ip|--dns_environment|--network_environment'
        ;;

    show_address)
        cmdOpts='--fqdn --dns_environment --network_environment'
        optsParam='--fqdn|--dns_environment|--network_environment'
        ;;

    update_address)
        cmdOpts='--fqdn --ip --comments --reverse_ptr --dns_environment --network_environment'
        optsParam='--fqdn|--ip|--comments|--reverse_ptr|--dns_environment|--network_environment'
        ;;

    show_ns_record)
        cmdOpts='--dns_domain --fqdn --all'
        optsParam='--dns_domain|--fqdn|--all'
        ;;

    add_ns_record)
        cmdOpts='--fqdn --dns_domain --comments'
        optsParam='--fqdn|--dns_domain|--comments'
        ;;

    del_ns_record)
        cmdOpts='--fqdn --dns_domain'
        optsParam='--fqdn|--dns_domain'
        ;;

    add_alias)
        cmdOpts='--fqdn --target --comments --dns_environment'
        optsParam='--fqdn|--target|--comments|--dns_environment'
        ;;

    del_alias)
        cmdOpts='--fqdn --dns_environment'
        optsParam='--fqdn|--dns_environment'
        ;;

    show_alias)
        cmdOpts='--fqdn --dns_environment'
        optsParam='--fqdn|--dns_environment'
        ;;

    update_alias)
        cmdOpts='--fqdn --target --comments --dns_environment'
        optsParam='--fqdn|--target|--comments|--dns_environment'
        ;;

    add_srv_record)
        cmdOpts='--service --protocol --dns_domain --priority --weight --target --port --comments --dns_environment'
        optsParam='--service|--protocol|--dns_domain|--priority|--weight|--target|--port|--comments|--dns_environment'
        ;;

    del_srv_record)
        cmdOpts='--service --protocol --dns_domain --dns_environment --target'
        optsParam='--service|--protocol|--dns_domain|--dns_environment|--target'
        ;;

    show_srv_record)
        cmdOpts='--service --protocol --dns_domain --dns_environment'
        optsParam='--service|--protocol|--dns_domain|--dns_environment'
        ;;

    update_srv_record)
        cmdOpts='--service --protocol --dns_domain --target --priority --weight --port --comments --dns_environment'
        optsParam='--service|--protocol|--dns_domain|--target|--priority|--weight|--port|--comments|--dns_environment'
        ;;

    search_dns)
        cmdOpts='--fqdn --shortname --record_type --ip --network --target --target_domain --dns_domain --primary_name --used --reverse_override --reverse_ptr --dns_environment --network_environment --fullinfo'
        optsParam='--fqdn|--shortname|--record_type|--ip|--network|--target|--target_domain|--dns_domain|--reverse_ptr|--dns_environment|--network_environment|--fullinfo'
        ;;

    search_rack)
        cmdOpts='--organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --all --row --column --fullinfo'
        optsParam='--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--all|--row|--column|--fullinfo'
        ;;

    map_dns_domain)
        cmdOpts='--dns_domain --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --position --comments'
        optsParam='--dns_domain|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--position|--comments'
        ;;

    unmap_dns_domain)
        cmdOpts='--dns_domain --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk'
        optsParam='--dns_domain|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk'
        ;;

    search_dns_domain_map)
        cmdOpts='--dns_domain --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --include_parents'
        optsParam='--dns_domain|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--include_parents'
        ;;

    search_audit)
        cmdOpts='--keyword --argument --username --command --before --after --return_code --limit --reverse_order'
        optsParam='--keyword|--argument|--username|--command|--before|--after|--return_code|--limit|--reverse_order'
        ;;

    show_reboot_schedule)
        cmdOpts='--cluster --hostname --all'
        optsParam='--cluster|--hostname|--all'
        ;;

    add_reboot_schedule)
        cmdOpts='--week --day --time --comments --hostname --cluster'
        optsParam='--week|--day|--time|--comments|--hostname|--cluster'
        ;;

    del_reboot_schedule)
        cmdOpts='--hostname --cluster'
        optsParam='--hostname|--cluster'
        ;;

    show_reboot_intervention)
        cmdOpts='--cluster --hostname --all'
        optsParam='--cluster|--hostname|--all'
        ;;

    add_reboot_intervention)
        cmdOpts='--expiry --justification --comments --start_time --hostname --cluster'
        optsParam='--expiry|--justification|--comments|--start_time|--hostname|--cluster'
        ;;

    del_reboot_intervention)
        cmdOpts='--hostname --cluster'
        optsParam='--hostname|--cluster'
        ;;

    add_filesystem)
        cmdOpts='--filesystem --type --mountpoint --blockdevice --bootmount --dumpfreq --fsckpass --options --comments --hostname --resourcegroup --cluster'
        optsParam='--filesystem|--type|--mountpoint|--blockdevice|--dumpfreq|--fsckpass|--options|--comments|--hostname|--resourcegroup|--cluster'
        ;;

    del_filesystem)
        cmdOpts='--filesystem --hostname --resourcegroup --cluster'
        optsParam='--filesystem|--hostname|--resourcegroup|--cluster'
        ;;

    show_filesystem)
        cmdOpts='--resourcegroup --cluster --hostname --filesystem --all'
        optsParam='--resourcegroup|--cluster|--hostname|--filesystem|--all'
        ;;

    add_share)
        cmdOpts='--share --comments --latency --hostname --resourcegroup --cluster'
        optsParam='--share|--comments|--latency|--hostname|--resourcegroup|--cluster'
        ;;

    del_share)
        cmdOpts='--share --hostname --resourcegroup --cluster'
        optsParam='--share|--hostname|--resourcegroup|--cluster'
        ;;

    show_share)
        cmdOpts='--share --hostname --resourcegroup --cluster --all'
        optsParam='--share|--hostname|--resourcegroup|--cluster|--all'
        ;;

    add_application)
        cmdOpts='--application --eonid --comments --resourcegroup --hostname --cluster'
        optsParam='--application|--eonid|--comments|--resourcegroup|--hostname|--cluster'
        ;;

    del_application)
        cmdOpts='--application --resourcegroup --hostname --cluster'
        optsParam='--application|--resourcegroup|--hostname|--cluster'
        ;;

    show_application)
        cmdOpts='--cluster --hostname --resourcegroup --application --eonid --all'
        optsParam='--cluster|--hostname|--resourcegroup|--application|--eonid|--all'
        ;;

    add_hostlink)
        cmdOpts='--hostlink --target --owner --group --comments --resourcegroup --hostname --cluster'
        optsParam='--hostlink|--target|--owner|--group|--comments|--resourcegroup|--hostname|--cluster'
        ;;

    del_hostlink)
        cmdOpts='--hostlink --resourcegroup --hostname --cluster'
        optsParam='--hostlink|--resourcegroup|--hostname|--cluster'
        ;;

    show_hostlink)
        cmdOpts='--cluster --hostname --resourcegroup --hostlink --all'
        optsParam='--cluster|--hostname|--resourcegroup|--hostlink|--all'
        ;;

    add_intervention)
        cmdOpts='--expiry --intervention --justification --comments --allowusers --allowgroups --disabled_actions --start_time --hostname --cluster'
        optsParam='--expiry|--intervention|--justification|--comments|--allowusers|--allowgroups|--disabled_actions|--start_time|--hostname|--cluster'
        ;;

    show_intervention)
        cmdOpts='--intervention --cluster --hostname --resourcegroup --all'
        optsParam='--intervention|--cluster|--hostname|--resourcegroup|--all'
        ;;

    del_intervention)
        cmdOpts='--intervention --hostname --cluster'
        optsParam='--intervention|--hostname|--cluster'
        ;;

    add_service_address)
        cmdOpts='--service_address --name --interfaces --resourcegroup --hostname --cluster --ip --network_environment --comments --map_to_primary'
        optsParam='--service_address|--name|--interfaces|--resourcegroup|--hostname|--cluster|--ip|--network_environment|--comments'
        ;;

    del_service_address)
        cmdOpts='--name --resourcegroup --hostname --cluster --keep_dns'
        optsParam='--name|--resourcegroup|--hostname|--cluster|--keep_dns'
        ;;

    show_service_address)
        cmdOpts='--cluster --hostname --resourcegroup --name --all'
        optsParam='--cluster|--hostname|--resourcegroup|--name|--all'
        ;;

    add_grn)
        cmdOpts='--grn --eon_id --disabled'
        optsParam='--grn|--eon_id|--disabled'
        ;;

    del_grn)
        cmdOpts='--grn --eon_id'
        optsParam='--grn|--eon_id'
        ;;

    update_grn)
        cmdOpts='--grn --eon_id --disabled --rename_to'
        optsParam='--grn|--eon_id|--rename_to'
        ;;

    show_grn)
        cmdOpts='--grn --eon_id --all'
        optsParam='--grn|--eon_id|--all'
        ;;

    refresh_grns)
        cmdOpts=''
        optsParam=''
        ;;

    map_grn)
        cmdOpts='--target --grn --eon_id --hostname --list --personality --archetype'
        optsParam='--target|--grn|--eon_id|--hostname|--list|--personality|--archetype'
        ;;

    unmap_grn)
        cmdOpts='--target --grn --eon_id --clearall --hostname --list --personality --archetype'
        optsParam='--target|--grn|--eon_id|--clearall|--hostname|--list|--personality|--archetype'
        ;;

    add_resourcegroup)
        cmdOpts='--resourcegroup --required_type --hostname --cluster'
        optsParam='--resourcegroup|--required_type|--hostname|--cluster'
        ;;

    del_resourcegroup)
        cmdOpts='--resourcegroup --hostname --cluster'
        optsParam='--resourcegroup|--hostname|--cluster'
        ;;

    show_resourcegroup)
        cmdOpts='--cluster --hostname --resourcegroup --all'
        optsParam='--cluster|--hostname|--resourcegroup|--all'
        ;;

    add_feature)
        cmdOpts='--feature --type --post_personality --comments'
        optsParam='--feature|--type|--post_personality|--comments'
        ;;

    del_feature)
        cmdOpts='--feature --type'
        optsParam='--feature|--type'
        ;;

    show_feature)
        cmdOpts='--feature --type --all'
        optsParam='--feature|--type|--all'
        ;;

    bind_feature)
        cmdOpts='--feature --personality --archetype --model --interface --vendor --justification'
        optsParam='--feature|--personality|--archetype|--model|--interface|--vendor|--justification'
        ;;

    unbind_feature)
        cmdOpts='--feature --personality --archetype --model --interface --vendor --justification --flush'
        optsParam='--feature|--personality|--archetype|--model|--interface|--vendor|--justification'
        ;;

    show_parameter)
        cmdOpts='--personality --archetype'
        optsParam='--personality|--archetype'
        ;;

    validate_parameter)
        cmdOpts='--personality --archetype'
        optsParam='--personality|--archetype'
        ;;

    add_parameter)
        cmdOpts='--personality --path --feature --archetype --interface --model --value --comments'
        optsParam='--personality|--path|--feature|--archetype|--interface|--model|--value|--comments'
        ;;

    update_parameter)
        cmdOpts='--personality --path --value --feature --archetype --interface --model --comments'
        optsParam='--personality|--path|--value|--feature|--archetype|--interface|--model|--comments'
        ;;

    del_parameter)
        cmdOpts='--personality --path --feature --archetype --interface --model'
        optsParam='--personality|--path|--feature|--archetype|--interface|--model'
        ;;

    show_diff)
        cmdOpts='--archetype --personality --other --other_archetype'
        optsParam='--archetype|--personality|--other|--other_archetype'
        ;;

    add_parameter_definition)
        cmdOpts='--archetype --template --feature --type --path --value_type --required --default --description --rebuild_required'
        optsParam='--archetype|--template|--feature|--type|--path|--value_type|--required|--default|--description|--rebuild_required'
        ;;

    del_parameter_definition)
        cmdOpts='--archetype --feature --type --path'
        optsParam='--archetype|--feature|--type|--path'
        ;;

    update_parameter_definition)
        cmdOpts='--archetype --feature --type --required --default --description --rebuild_required --path'
        optsParam='--archetype|--feature|--type|--required|--default|--description|--rebuild_required|--path'
        ;;

    search_parameter_definition)
        cmdOpts='--archetype --feature --type'
        optsParam='--archetype|--feature|--type'
        ;;

    search_parameter)
        cmdOpts='--archetype --path'
        optsParam='--archetype|--path'
        ;;

    add_vlan)
        cmdOpts='--vlan --name --vlan_type'
        optsParam='--vlan|--name|--vlan_type'
        ;;

    del_vlan)
        cmdOpts='--vlan'
        optsParam='--vlan'
        ;;

    show_vlan)
        cmdOpts='--vlan --all'
        optsParam='--vlan|--all'
        ;;

    show_bunker_violations)
        cmdOpts=''
        optsParam=''
        ;;
    esac

    # append global options
    # cmdOpts="$cmdOpts $globalOpts"
    # optsParam="$optsParam|$globalParams"

    # take out options already given
    for (( i = 2; i<= $COMP_CWORD-1; ++i)); do
        opt=${COMP_WORDS[$i]}

        case $opt in
            --*) optBase=${opt/=*/} ;;
            -*)  optBase=${opt:0:2} ;;
        esac

        cmdOpts=" $cmdOpts "
        cmdOpts=${cmdOpts/ ${optBase} / }

        # skip next option if this one requires a parameter
        if [[ "$opt" =~ "$optsParam" ]]; then
            ((++i))
        fi
    done

    COMPREPLY=( $( compgen -W "$cmdOpts" -- $cur ) )

    return 0
}

complete -F _aq -o default aq
