-- Move ip and network_id from system to future_a_record
ALTER TABLE system DROP CONSTRAINT "SYSTEM_PT_UK";
ALTER TABLE future_a_record ADD ip INTEGER;
ALTER TABLE future_a_record ADD network_id INTEGER;
UPDATE future_a_record SET ip = (SELECT ip FROM system WHERE system.id = future_a_record.system_id);
UPDATE future_a_record SET network_id = (SELECT network_id FROM system WHERE system.id = future_a_record.system_id);
ALTER TABLE future_a_record
	MODIFY (ip CONSTRAINT "FUTURE_A_RECORD_IP_NN" NOT NULL);
ALTER TABLE future_a_record
	ADD CONSTRAINT "FUTURE_A_RECORD_NETWORK_FK"
		FOREIGN KEY (network_id)
		REFERENCES network (id)
		ON DELETE SET NULL;
ALTER TABLE system DROP COLUMN ip;
ALTER TABLE system DROP COLUMN network_id;

CREATE SEQUENCE dns_environment_id_seq;
CREATE TABLE dns_environment (
	id INTEGER CONSTRAINT "DNS_ENVIRONMENT_ID_NN" NOT NULL,
	name VARCHAR2(32 CHAR) CONSTRAINT "DNS_ENVIRONMENT_NAME_NN" NOT NULL,
	creation_date DATE CONSTRAINT "DNS_ENVIRONMENT_CR_DATE_NN" NOT NULL,
	comments VARCHAR2(255 CHAR),
	CONSTRAINT "DNS_ENVIRONMENT_PK" PRIMARY KEY (id),
	CONSTRAINT "DNS_ENVIRONMENT_NAME_UK" UNIQUE (name)
);

INSERT INTO dns_environment (id, name, creation_date)
	VALUES (dns_environment_id_seq.nextval, 'internal', CURRENT_DATE);
INSERT INTO dns_environment (id, name, creation_date)
	VALUES (dns_environment_id_seq.nextval, 'external', CURRENT_DATE);
INSERT INTO dns_environment (id, name, creation_date)
	VALUES (dns_environment_id_seq.nextval, 'corpseg', CURRENT_DATE);

ALTER TABLE system RENAME TO dns_record;
RENAME system_seq TO dns_record_id_seq;
ALTER TABLE dns_record MODIFY (name VARCHAR2(63 CHAR));
ALTER TABLE dns_record RENAME COLUMN system_type TO dns_record_type;
ALTER TABLE dns_record MODIFY (dns_record_type VARCHAR2(32 CHAR));
ALTER TABLE dns_record MODIFY (comments VARCHAR2(255 CHAR));
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_ID_NN" TO "DNS_RECORD_ID_NN";
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_NAME_NN" TO "DNS_RECORD_NAME_NN";
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_SYSTEM_TYPE_NN" TO "DNS_RECORD_DNS_RECORD_TYPE_NN";
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_DNS_DOMAIN_ID_NN" TO "DNS_RECORD_DNS_DOMAIN_ID_NN";
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_CR_DATE_NN" TO "DNS_RECORD_CR_DATE_NN";
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_PK" TO "DNS_RECORD_PK";
ALTER INDEX "SYSTEM_PK" RENAME TO "DNS_RECORD_PK";
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_DNS_NAME_UK" TO "DNS_RECORD_NAME_DOMAIN_UK";
ALTER INDEX "SYSTEM_DNS_NAME_UK" RENAME TO "DNS_RECORD_NAME_DOMAIN_UK";
ALTER TABLE dns_record RENAME CONSTRAINT "SYSTEM_DNS_FK" TO "DNS_RECORD_DNS_DOMAIN_FK";

ALTER TABLE future_a_record RENAME COLUMN system_id TO dns_record_id;
ALTER TABLE future_a_record RENAME CONSTRAINT "FUTURE_A_RECORD_SYSTEM_ID_NN" TO "FUTURE_A_RECORD_DNSREC_ID_NN";
ALTER TABLE future_a_record RENAME CONSTRAINT "FUTURE_A_RECORD_SYSTEM_FK" TO "FUTURE_A_RECORD_DNS_RECORD_FK";

ALTER TABLE dynamic_stub RENAME COLUMN system_id TO dns_record_id;
ALTER TABLE dynamic_stub RENAME CONSTRAINT "DYNAMIC_STUB_SYSTEM_ID_NN" TO "DYNAMIC_STUB_DNS_RECORD_ID_NN";

ALTER TABLE reserved_name RENAME COLUMN system_id TO dns_record_id;
ALTER TABLE reserved_name RENAME CONSTRAINT "RESERVED_NAME_SYSTEM_ID_NN" TO "RESERVED_NAME_DNS_RECORD_ID_NN";
ALTER TABLE reserved_name RENAME CONSTRAINT "RESERVED_NAME_SYSTEM_FK" TO "RESERVED_NAME_DNS_RECORD_FK";

ALTER TABLE dns_record ADD dns_environment_id INTEGER;
UPDATE dns_record SET dns_environment_id = (SELECT id FROM dns_environment WHERE name = 'internal');
ALTER TABLE dns_record
	MODIFY (dns_environment_id CONSTRAINT "DNS_RECORD_DNS_ENV_ID_NN" NOT NULL);
ALTER TABLE dns_record
	ADD CONSTRAINT "DNS_RECORD_DNS_ENV_FK" FOREIGN KEY (dns_environment_id) REFERENCES dns_environment (id);
ALTER TABLE dns_record DROP CONSTRAINT "DNS_RECORD_NAME_DOMAIN_UK";
DROP INDEX "DNS_RECORD_NAME_DOMAIN_UK";
ALTER TABLE dns_record
	ADD CONSTRAINT "DNS_RECORD_NAME_DOMAIN_ENV_UK" UNIQUE (name, dns_domain_id, dns_environment_id);

ALTER TABLE future_a_record RENAME TO a_record;
UPDATE dns_record SET dns_record_type = 'a_record' WHERE dns_record_type = 'future_a_record';
ALTER TABLE a_record RENAME CONSTRAINT "FUTURE_A_RECORD_IP_NN" TO "A_RECORD_IP_NN";
ALTER TABLE a_record RENAME CONSTRAINT "FUTURE_A_RECORD_DNSREC_ID_NN" TO "A_RECORD_DNS_RECORD_ID_NN";
ALTER TABLE a_record RENAME CONSTRAINT "FUTURE_A_RECORD_DNS_RECORD_FK" TO "A_RECORD_DNS_RECORD_FK";
ALTER TABLE a_record RENAME CONSTRAINT "FUTURE_A_RECORD_PK" TO "A_RECORD_PK";
ALTER TABLE a_record RENAME CONSTRAINT "FUTURE_A_RECORD_NETWORK_FK" TO "A_RECORD_NETWORK_FK";

ALTER INDEX "FUTURE_A_RECORD_PK" RENAME TO "A_RECORD_PK";

ALTER TABLE dynamic_stub RENAME CONSTRAINT "DYNAMIC_STUB_FARECORD_FK" TO "DYNAMIC_STUB_ARECORD_FK";

CREATE SEQUENCE dns_map_id_seq;
CREATE TABLE dns_map (
	id INTEGER CONSTRAINT "DNS_MAP_ID_NN" NOT NULL,
	location_id INTEGER CONSTRAINT "DNS_MAP_LOCATION_ID_NN" NOT NULL,
	dns_domain_id INTEGER CONSTRAINT "DNS_MAP_DNS_DOMAIN_ID_NN" NOT NULL,
	position INTEGER CONSTRAINT "DNS_MAP_POSITION_NN" NOT NULL,
	creation_date DATE CONSTRAINT "DNS_MAP_CR_DATE_NN" NOT NULL,
	comments VARCHAR2(255 CHAR),
	CONSTRAINT "DNS_MAP_PK" PRIMARY KEY (id),
	CONSTRAINT "DNS_MAP_LOC_DNS_DOM_UK" UNIQUE (location_id, dns_domain_id),
	CONSTRAINT "DNS_MAP_LOCATION_FK" FOREIGN KEY (location_id) REFERENCES location (id) ON DELETE CASCADE,
	CONSTRAINT "DNS_MAP_DNS_DOMAIN_FK" FOREIGN KEY (dns_domain_id) REFERENCES dns_domain (id)
);

ALTER TABLE router_address ADD dns_environment_id INTEGER;
UPDATE router_address SET dns_environment_id = (SELECT id FROM dns_environment WHERE name = 'internal');
ALTER TABLE router_address
	MODIFY (dns_environment_id CONSTRAINT "ROUTER_ADDRESS_DNS_ENV_ID_NN" NOT NULL);
ALTER TABLE router_address
	ADD CONSTRAINT "ROUTER_ADDRESS_DNS_ENV_FK" FOREIGN KEY (dns_environment_id) REFERENCES dns_environment (id);

ALTER TABLE address_assignment ADD dns_environment_id INTEGER;
UPDATE address_assignment SET dns_environment_id = (SELECT id FROM dns_environment WHERE name = 'internal');
ALTER TABLE address_assignment
	MODIFY (dns_environment_id CONSTRAINT "ADDR_ASSIGN_DNS_ENV_ID_NN" NOT NULL);
ALTER TABLE address_assignment
	ADD CONSTRAINT "ADDR_ASSIGN_DNS_ENV_FK" FOREIGN KEY (dns_environment_id) REFERENCES dns_environment (id);

QUIT;
