CREATE TABLE "XTN" (
    "XTN_ID"      CHAR(32) CONSTRAINT "XTN_XTN_ID_NN" NOT NULL,
    "USERNAME"    VARCHAR2(65) CONSTRAINT "XTN_USERNAME_NN" NOT NULL,
    "COMMAND"     VARCHAR2(64) CONSTRAINT "XTN_COMMAND_NN" NOT NULL,
    "IS_READONLY" NUMBER(*,0) CONSTRAINT "XTN_IS_READONLY_NN" NOT NULL,
    "START_TIME"  DATE CONSTRAINT "XTN_START_TIME_NN" NOT NULL,
    CONSTRAINT "XTN_IS_READONLY" CHECK (is_readonly IN (0, 1)),
    CONSTRAINT "XTN_PK" PRIMARY KEY ("XTN_ID")) COMPRESS;


CREATE TABLE "XTN_DETAIL" (
    "XTN_ID" CHAR(32) CONSTRAINT "XTN_DETAIL_XTN_ID_NN" NOT NULL,
    "NAME"   VARCHAR2(255) CONSTRAINT "XTN_DETAIL_NAME_NN" NOT NULL,
    "VALUE"  VARCHAR2(255) CONSTRAINT "XTN_DETAIL_VALUE_NN" NOT NULL,
    CONSTRAINT "XTN_DTL_PK" PRIMARY KEY ("XTN_ID", "NAME", "VALUE"),
    CONSTRAINT "XTN_DTL_XTN_FK" FOREIGN KEY ("XTN_ID")
        REFERENCES "XTN" ("XTN_ID")) COMPRESS;


CREATE TABLE "XTN_END" (
    "XTN_ID" CHAR(32) CONSTRAINT "XTN_END_XTN_ID_NN" NOT NULL,
    "RETURN_CODE" NUMBER(*,0) CONSTRAINT "XTN_END_RETURN_CODE_NN" NOT NULL,
    "END_TIME" DATE CONSTRAINT "XTN_END_END_TIME_NN" NOT NULL,
    CONSTRAINT "XTN_END_PK" PRIMARY KEY ("XTN_ID"),
    CONSTRAINT "XTN_END_XTN_FK" FOREIGN KEY("XTN_ID")
        REFERENCES "XTN" ("XTN_ID")) COMPRESS;


CREATE INDEX "XTN_START_TIME_IDX" ON "XTN" ("START_TIME" DESC);
CREATE INDEX "XTN_COMMAND_IDX" ON "XTN" ("COMMAND") COMPRESS;
CREATE INDEX "XTN_USERNAME_IDX" ON "XTN" ("USERNAME") COMPRESS;
CREATE BITMAP INDEX "XTN_ISREADONLY_IDX" ON "XTN" ("IS_READONLY" ASC);
CREATE INDEX "XTN_DTL_VALUE_IDX" ON "XTN_DETAIL" ("VALUE") COMPRESS;
CREATE INDEX "XTN_DTL_NAME_IDX" ON "XTN_DETAIL" ("NAME") COMPRESS;
CREATE INDEX "XTN_END_RETURN_CODE_IDX" ON "XTN_END" ("RETURN_CODE") COMPRESS;


GRANT SELECT, INSERT on XTN to cdb;
GRANT SELECT, INSERT on XTN_DETAIL to cdb;
GRANT SELECT, INSERT on XTN_END to cdb;


COMMENT ON COLUMN XTN.START_TIME IS 'The start_date column is forced to UTC by the broker';
COMMENT ON COLUMN XTN_END.END_TIME IS 'The end_date column is forced to UTC by the broker';

QUIT;
